@@ -13,12 +13,12 @@
  */
 
 #include "libdefs.h"
-#include "gosthash.h"
+#include "mhash_gost.h"
 /*
    lookup tables : each of these has two rotated 4-bit S-Boxes 
  */
 
-static word32 gost_sbox_1[256] = {
+static const word32 gost_sbox_1[256] = {
 
 	0x72000UL, 0x75000UL, 0x74800UL, 0x71000UL, 0x76800UL,
 	0x74000UL, 0x70000UL, 0x77000UL, 0x73000UL, 0x75800UL,
@@ -55,13 +55,13 @@ static word32 gost_sbox_1[256] = {
 	0x42000UL, 0x45000UL, 0x44800UL, 0x41000UL, 0x46800UL,
 	0x44000UL, 0x40000UL, 0x47000UL, 0x43000UL, 0x45800UL,
 	0x40800UL, 0x46000UL, 0x43800UL, 0x47800UL, 0x42800UL,
-	0x41800UL, 0xA000UL, 0xD000UL, 0xC800UL, 0x9000UL,
-	0xE800UL, 0xC000UL, 0x8000UL, 0xF000UL, 0xB000UL,
-	0xD800UL, 0x8800UL, 0xE000UL, 0xB800UL, 0xF800UL,
-	0xA800UL, 0x9800UL, 0x2000UL, 0x5000UL, 0x4800UL,
-	0x1000UL, 0x6800UL, 0x4000UL, 0x0UL, 0x7000UL,
-	0x3000UL, 0x5800UL, 0x800UL, 0x6000UL, 0x3800UL,
-	0x7800UL, 0x2800UL, 0x1800UL, 0x3A000UL, 0x3D000UL,
+	0x41800UL, 0xA000UL,  0xD000UL,  0xC800UL,  0x9000UL,
+	0xE800UL,  0xC000UL,  0x8000UL,  0xF000UL,  0xB000UL,
+	0xD800UL,  0x8800UL,  0xE000UL,  0xB800UL,  0xF800UL,
+	0xA800UL,  0x9800UL,  0x2000UL,  0x5000UL,  0x4800UL,
+	0x1000UL,  0x6800UL,  0x4000UL,  0x0UL,     0x7000UL,
+	0x3000UL,  0x5800UL,  0x800UL,   0x6000UL,  0x3800UL,
+	0x7800UL,  0x2800UL,  0x1800UL,  0x3A000UL, 0x3D000UL,
 	0x3C800UL, 0x39000UL, 0x3E800UL, 0x3C000UL, 0x38000UL,
 	0x3F000UL, 0x3B000UL, 0x3D800UL, 0x38800UL, 0x3E000UL,
 	0x3B800UL, 0x3F800UL, 0x3A800UL, 0x39800UL, 0x2A000UL,
@@ -73,7 +73,7 @@ static word32 gost_sbox_1[256] = {
 	0x48800UL, 0x4E000UL, 0x4B800UL, 0x4F800UL, 0x4A800UL,
 	0x49800UL,
 };
-static word32 gost_sbox_2[256] = {
+static const word32 gost_sbox_2[256] = {
 
 	0x3A80000UL, 0x3C00000UL, 0x3880000UL, 0x3E80000UL, 0x3D00000UL,
 	0x3980000UL, 0x3A00000UL, 0x3900000UL, 0x3F00000UL, 0x3F80000UL,
@@ -84,13 +84,13 @@ static word32 gost_sbox_2[256] = {
 	0x6C80000UL, 0x6D80000UL, 0x5280000UL, 0x5400000UL, 0x5080000UL,
 	0x5680000UL, 0x5500000UL, 0x5180000UL, 0x5200000UL, 0x5100000UL,
 	0x5700000UL, 0x5780000UL, 0x5600000UL, 0x5380000UL, 0x5300000UL,
-	0x5000000UL, 0x5480000UL, 0x5580000UL, 0xA80000UL, 0xC00000UL,
-	0x880000UL, 0xE80000UL, 0xD00000UL, 0x980000UL, 0xA00000UL,
-	0x900000UL, 0xF00000UL, 0xF80000UL, 0xE00000UL, 0xB80000UL,
-	0xB00000UL, 0x800000UL, 0xC80000UL, 0xD80000UL, 0x280000UL,
-	0x400000UL, 0x80000UL, 0x680000UL, 0x500000UL, 0x180000UL,
-	0x200000UL, 0x100000UL, 0x700000UL, 0x780000UL, 0x600000UL,
-	0x380000UL, 0x300000UL, 0x0UL, 0x480000UL, 0x580000UL,
+	0x5000000UL, 0x5480000UL, 0x5580000UL, 0xA80000UL,  0xC00000UL,
+	0x880000UL,  0xE80000UL,  0xD00000UL,  0x980000UL,  0xA00000UL,
+	0x900000UL,  0xF00000UL,  0xF80000UL,  0xE00000UL,  0xB80000UL,
+	0xB00000UL,  0x800000UL,  0xC80000UL,  0xD80000UL,  0x280000UL,
+	0x400000UL,  0x80000UL,   0x680000UL,  0x500000UL,  0x180000UL,
+	0x200000UL,  0x100000UL,  0x700000UL,  0x780000UL,  0x600000UL,
+	0x380000UL,  0x300000UL,  0x0UL,       0x480000UL,  0x580000UL,
 	0x4280000UL, 0x4400000UL, 0x4080000UL, 0x4680000UL, 0x4500000UL,
 	0x4180000UL, 0x4200000UL, 0x4100000UL, 0x4700000UL, 0x4780000UL,
 	0x4600000UL, 0x4380000UL, 0x4300000UL, 0x4000000UL, 0x4480000UL,
@@ -128,119 +128,88 @@ static word32 gost_sbox_2[256] = {
 	0x1E00000UL, 0x1B80000UL, 0x1B00000UL, 0x1800000UL, 0x1C80000UL,
 	0x1D80000UL,
 };
-static word32 gost_sbox_3[256] = {
+static const word32 gost_sbox_3[256] = {
 
 	0x30000002UL, 0x60000002UL, 0x38000002UL, 0x8000002UL,
-	0x28000002UL,
-	0x78000002UL, 0x68000002UL, 0x40000002UL, 0x20000002UL,
-	0x50000002UL,
-	0x48000002UL, 0x70000002UL, 0x2UL, 0x18000002UL, 0x58000002UL,
-	0x10000002UL, 0xB0000005UL, 0xE0000005UL, 0xB8000005UL,
-	0x88000005UL,
+	0x28000002UL, 0x78000002UL, 0x68000002UL, 0x40000002UL, 
+	0x20000002UL, 0x50000002UL, 0x48000002UL, 0x70000002UL, 
+	0x2UL,        0x18000002UL, 0x58000002UL, 0x10000002UL, 
+	0xB0000005UL, 0xE0000005UL, 0xB8000005UL, 0x88000005UL,
 	0xA8000005UL, 0xF8000005UL, 0xE8000005UL, 0xC0000005UL,
-	0xA0000005UL,
-	0xD0000005UL, 0xC8000005UL, 0xF0000005UL, 0x80000005UL,
-	0x98000005UL,
-	0xD8000005UL, 0x90000005UL, 0x30000005UL, 0x60000005UL,
-	0x38000005UL,
-	0x8000005UL, 0x28000005UL, 0x78000005UL, 0x68000005UL,
-	0x40000005UL,
-	0x20000005UL, 0x50000005UL, 0x48000005UL, 0x70000005UL, 0x5UL,
-	0x18000005UL, 0x58000005UL, 0x10000005UL, 0x30000000UL,
-	0x60000000UL,
-	0x38000000UL, 0x8000000UL, 0x28000000UL, 0x78000000UL,
-	0x68000000UL,
-	0x40000000UL, 0x20000000UL, 0x50000000UL, 0x48000000UL,
-	0x70000000UL,
-	0x0UL, 0x18000000UL, 0x58000000UL, 0x10000000UL, 0xB0000003UL,
-	0xE0000003UL, 0xB8000003UL, 0x88000003UL, 0xA8000003UL,
-	0xF8000003UL,
-	0xE8000003UL, 0xC0000003UL, 0xA0000003UL, 0xD0000003UL,
-	0xC8000003UL,
-	0xF0000003UL, 0x80000003UL, 0x98000003UL, 0xD8000003UL,
-	0x90000003UL,
+	0xA0000005UL, 0xD0000005UL, 0xC8000005UL, 0xF0000005UL, 
+	0x80000005UL, 0x98000005UL, 0xD8000005UL, 0x90000005UL, 
+	0x30000005UL, 0x60000005UL, 0x38000005UL, 0x8000005UL, 
+	0x28000005UL, 0x78000005UL, 0x68000005UL, 0x40000005UL,
+	0x20000005UL, 0x50000005UL, 0x48000005UL, 0x70000005UL, 
+	0x5UL,        0x18000005UL, 0x58000005UL, 0x10000005UL, 
+	0x30000000UL, 0x60000000UL, 0x38000000UL, 0x8000000UL, 
+	0x28000000UL, 0x78000000UL, 0x68000000UL, 0x40000000UL, 
+	0x20000000UL, 0x50000000UL, 0x48000000UL, 0x70000000UL,
+	0x0UL,        0x18000000UL, 0x58000000UL, 0x10000000UL, 
+	0xB0000003UL, 0xE0000003UL, 0xB8000003UL, 0x88000003UL, 
+	0xA8000003UL, 0xF8000003UL, 0xE8000003UL, 0xC0000003UL, 
+	0xA0000003UL, 0xD0000003UL, 0xC8000003UL, 0xF0000003UL, 
+	0x80000003UL, 0x98000003UL, 0xD8000003UL, 0x90000003UL,
 	0x30000001UL, 0x60000001UL, 0x38000001UL, 0x8000001UL,
-	0x28000001UL,
-	0x78000001UL, 0x68000001UL, 0x40000001UL, 0x20000001UL,
-	0x50000001UL,
-	0x48000001UL, 0x70000001UL, 0x1UL, 0x18000001UL, 0x58000001UL,
-	0x10000001UL, 0xB0000000UL, 0xE0000000UL, 0xB8000000UL,
-	0x88000000UL,
+	0x28000001UL, 0x78000001UL, 0x68000001UL, 0x40000001UL, 
+	0x20000001UL, 0x50000001UL, 0x48000001UL, 0x70000001UL, 
+	0x1UL,        0x18000001UL, 0x58000001UL, 0x10000001UL, 
+	0xB0000000UL, 0xE0000000UL, 0xB8000000UL, 0x88000000UL,
 	0xA8000000UL, 0xF8000000UL, 0xE8000000UL, 0xC0000000UL,
-	0xA0000000UL,
-	0xD0000000UL, 0xC8000000UL, 0xF0000000UL, 0x80000000UL,
-	0x98000000UL,
-	0xD8000000UL, 0x90000000UL, 0xB0000006UL, 0xE0000006UL,
-	0xB8000006UL,
-	0x88000006UL, 0xA8000006UL, 0xF8000006UL, 0xE8000006UL,
-	0xC0000006UL,
+	0xA0000000UL, 0xD0000000UL, 0xC8000000UL, 0xF0000000UL, 
+	0x80000000UL, 0x98000000UL, 0xD8000000UL, 0x90000000UL, 
+	0xB0000006UL, 0xE0000006UL, 0xB8000006UL, 0x88000006UL, 
+	0xA8000006UL, 0xF8000006UL, 0xE8000006UL, 0xC0000006UL,
 	0xA0000006UL, 0xD0000006UL, 0xC8000006UL, 0xF0000006UL,
-	0x80000006UL,
-	0x98000006UL, 0xD8000006UL, 0x90000006UL, 0xB0000001UL,
-	0xE0000001UL,
-	0xB8000001UL, 0x88000001UL, 0xA8000001UL, 0xF8000001UL,
-	0xE8000001UL,
-	0xC0000001UL, 0xA0000001UL, 0xD0000001UL, 0xC8000001UL,
-	0xF0000001UL,
+	0x80000006UL, 0x98000006UL, 0xD8000006UL, 0x90000006UL, 
+	0xB0000001UL, 0xE0000001UL, 0xB8000001UL, 0x88000001UL, 
+	0xA8000001UL, 0xF8000001UL, 0xE8000001UL, 0xC0000001UL, 
+	0xA0000001UL, 0xD0000001UL, 0xC8000001UL, 0xF0000001UL,
 	0x80000001UL, 0x98000001UL, 0xD8000001UL, 0x90000001UL,
-	0x30000003UL,
-	0x60000003UL, 0x38000003UL, 0x8000003UL, 0x28000003UL,
-	0x78000003UL,
-	0x68000003UL, 0x40000003UL, 0x20000003UL, 0x50000003UL,
-	0x48000003UL,
-	0x70000003UL, 0x3UL, 0x18000003UL, 0x58000003UL, 0x10000003UL,
+	0x30000003UL, 0x60000003UL, 0x38000003UL, 0x8000003UL, 
+	0x28000003UL, 0x78000003UL, 0x68000003UL, 0x40000003UL, 
+	0x20000003UL, 0x50000003UL, 0x48000003UL, 0x70000003UL, 
+	0x3UL,        0x18000003UL, 0x58000003UL, 0x10000003UL,
 	0x30000004UL, 0x60000004UL, 0x38000004UL, 0x8000004UL,
-	0x28000004UL,
-	0x78000004UL, 0x68000004UL, 0x40000004UL, 0x20000004UL,
-	0x50000004UL,
-	0x48000004UL, 0x70000004UL, 0x4UL, 0x18000004UL, 0x58000004UL,
-	0x10000004UL, 0xB0000002UL, 0xE0000002UL, 0xB8000002UL,
-	0x88000002UL,
+	0x28000004UL, 0x78000004UL, 0x68000004UL, 0x40000004UL, 
+	0x20000004UL, 0x50000004UL, 0x48000004UL, 0x70000004UL, 
+	0x4UL,        0x18000004UL, 0x58000004UL, 0x10000004UL, 
+	0xB0000002UL, 0xE0000002UL, 0xB8000002UL, 0x88000002UL,
 	0xA8000002UL, 0xF8000002UL, 0xE8000002UL, 0xC0000002UL,
-	0xA0000002UL,
-	0xD0000002UL, 0xC8000002UL, 0xF0000002UL, 0x80000002UL,
-	0x98000002UL,
-	0xD8000002UL, 0x90000002UL, 0xB0000004UL, 0xE0000004UL,
-	0xB8000004UL,
-	0x88000004UL, 0xA8000004UL, 0xF8000004UL, 0xE8000004UL,
-	0xC0000004UL,
+	0xA0000002UL, 0xD0000002UL, 0xC8000002UL, 0xF0000002UL, 
+	0x80000002UL, 0x98000002UL, 0xD8000002UL, 0x90000002UL, 
+	0xB0000004UL, 0xE0000004UL, 0xB8000004UL, 0x88000004UL, 
+	0xA8000004UL, 0xF8000004UL, 0xE8000004UL, 0xC0000004UL,
 	0xA0000004UL, 0xD0000004UL, 0xC8000004UL, 0xF0000004UL,
-	0x80000004UL,
-	0x98000004UL, 0xD8000004UL, 0x90000004UL, 0x30000006UL,
-	0x60000006UL,
-	0x38000006UL, 0x8000006UL, 0x28000006UL, 0x78000006UL,
-	0x68000006UL,
-	0x40000006UL, 0x20000006UL, 0x50000006UL, 0x48000006UL,
-	0x70000006UL,
-	0x6UL, 0x18000006UL, 0x58000006UL, 0x10000006UL, 0xB0000007UL,
-	0xE0000007UL, 0xB8000007UL, 0x88000007UL, 0xA8000007UL,
-	0xF8000007UL,
-	0xE8000007UL, 0xC0000007UL, 0xA0000007UL, 0xD0000007UL,
-	0xC8000007UL,
-	0xF0000007UL, 0x80000007UL, 0x98000007UL, 0xD8000007UL,
-	0x90000007UL,
+	0x80000004UL, 0x98000004UL, 0xD8000004UL, 0x90000004UL, 
+	0x30000006UL, 0x60000006UL, 0x38000006UL, 0x8000006UL, 
+	0x28000006UL, 0x78000006UL, 0x68000006UL, 0x40000006UL, 
+	0x20000006UL, 0x50000006UL, 0x48000006UL, 0x70000006UL,
+	0x6UL,        0x18000006UL, 0x58000006UL, 0x10000006UL, 
+	0xB0000007UL, 0xE0000007UL, 0xB8000007UL, 0x88000007UL, 
+	0xA8000007UL, 0xF8000007UL, 0xE8000007UL, 0xC0000007UL, 
+	0xA0000007UL, 0xD0000007UL, 0xC8000007UL, 0xF0000007UL, 
+	0x80000007UL, 0x98000007UL, 0xD8000007UL, 0x90000007UL,
 	0x30000007UL, 0x60000007UL, 0x38000007UL, 0x8000007UL,
-	0x28000007UL,
-	0x78000007UL, 0x68000007UL, 0x40000007UL, 0x20000007UL,
-	0x50000007UL,
-	0x48000007UL, 0x70000007UL, 0x7UL, 0x18000007UL, 0x58000007UL,
-	0x10000007UL,
+	0x28000007UL, 0x78000007UL, 0x68000007UL, 0x40000007UL, 
+	0x20000007UL, 0x50000007UL, 0x48000007UL, 0x70000007UL, 
+	0x7UL,        0x18000007UL, 0x58000007UL, 0x10000007UL,
 };
-static word32 gost_sbox_4[256] = {
+static const word32 gost_sbox_4[256] = {
 
-	0xE8UL, 0xD8UL, 0xA0UL, 0x88UL, 0x98UL,
-	0xF8UL, 0xA8UL, 0xC8UL, 0x80UL, 0xD0UL,
-	0xF0UL, 0xB8UL, 0xB0UL, 0xC0UL, 0x90UL,
-	0xE0UL, 0x7E8UL, 0x7D8UL, 0x7A0UL, 0x788UL,
+	0xE8UL,  0xD8UL,  0xA0UL,  0x88UL,  0x98UL,
+	0xF8UL,  0xA8UL,  0xC8UL,  0x80UL,  0xD0UL,
+	0xF0UL,  0xB8UL,  0xB0UL,  0xC0UL,  0x90UL,
+	0xE0UL,  0x7E8UL, 0x7D8UL, 0x7A0UL, 0x788UL,
 	0x798UL, 0x7F8UL, 0x7A8UL, 0x7C8UL, 0x780UL,
 	0x7D0UL, 0x7F0UL, 0x7B8UL, 0x7B0UL, 0x7C0UL,
 	0x790UL, 0x7E0UL, 0x6E8UL, 0x6D8UL, 0x6A0UL,
 	0x688UL, 0x698UL, 0x6F8UL, 0x6A8UL, 0x6C8UL,
 	0x680UL, 0x6D0UL, 0x6F0UL, 0x6B8UL, 0x6B0UL,
-	0x6C0UL, 0x690UL, 0x6E0UL, 0x68UL, 0x58UL,
-	0x20UL, 0x8UL, 0x18UL, 0x78UL, 0x28UL,
-	0x48UL, 0x0UL, 0x50UL, 0x70UL, 0x38UL,
-	0x30UL, 0x40UL, 0x10UL, 0x60UL, 0x2E8UL,
+	0x6C0UL, 0x690UL, 0x6E0UL, 0x68UL,  0x58UL,
+	0x20UL,  0x8UL,   0x18UL,  0x78UL,  0x28UL,
+	0x48UL,  0x0UL,   0x50UL,  0x70UL,  0x38UL,
+	0x30UL,  0x40UL,  0x10UL,  0x60UL,  0x2E8UL,
 	0x2D8UL, 0x2A0UL, 0x288UL, 0x298UL, 0x2F8UL,
 	0x2A8UL, 0x2C8UL, 0x280UL, 0x2D0UL, 0x2F0UL,
 	0x2B8UL, 0x2B0UL, 0x2C0UL, 0x290UL, 0x2E0UL,
@@ -300,7 +269,7 @@ void gosthash_init()
 	   4-bit S-Boxes 
 	 */
 
-	word32 sbox[8][16] = {
+	const word32 sbox[8][16] = {
 		{4, 10, 9, 2, 13, 8, 0, 14, 6, 11, 1, 12, 7, 15, 5, 3},
 		{14, 11, 4, 12, 6, 13, 15, 10, 2, 3, 8, 1, 0, 7, 5, 9},
 		{5, 8, 1, 13, 10, 3, 4, 2, 14, 15, 12, 7, 6, 0, 9, 11},
