@@ -20,7 +20,7 @@
  */
 
 
-/* $Id: mhash.c,v 1.28 2002/02/10 10:10:08 nmav Exp $ */
+/* $Id: mhash.c,v 1.30 2002/05/17 11:26:34 nmav Exp $ */
 
 #include <stdlib.h>
 
@@ -40,7 +40,7 @@
 #include "mhash_ripemd.h"
 #include "mhash_sha256.h"
 #include "mhash_adler32.h"
-#include "gosthash.h"
+#include "mhash_gost.h"
 
 /* 19/03/2000 Changes for better thread handling --nikos */
 
@@ -56,7 +56,7 @@ struct mhash_hash_entry {
 	size_t hash_pblock;
 };
 
-static mhash_hash_entry algorithms[] = {
+static const mhash_hash_entry algorithms[] = {
 	MHASH_ENTRY(MHASH_CRC32, 4, 0),
 	MHASH_ENTRY(MHASH_ADLER32, 4, 0),
 	MHASH_ENTRY(MHASH_MD5, 16, 64),
@@ -78,7 +78,7 @@ static mhash_hash_entry algorithms[] = {
 };
 
 #define MHASH_LOOP(b) \
-	mhash_hash_entry *p; \
+	const mhash_hash_entry *p; \
 	for(p = algorithms; p->name != NULL; p++) { b ; }
 
 #define MHASH_ALG_LOOP(a) \
@@ -761,4 +761,3 @@ WIN32DLL_DEFINE MHASH mhash_restore_state_mem(void* _mem)
 		free( ret);
 		return MHASH_FAILED;
 }
-
